@@ -629,17 +629,6 @@ main (int argc, char **argv)
 	for (i = 0; tests[i].test_name; i++) {
 		gchar *testpath;
 
-#ifndef HAVE_LIBICU
-		/* Skip tests which fail collation tests and are known
-		 * to do so. For more details see:
-		 *
-		 * https://bugzilla.gnome.org/show_bug.cgi?id=636074
-		 */
-		if (strcmp (tests[i].test_name, "functions/functions-xpath-2") == 0) {
-			continue;
-		}
-#endif
-
 		testpath = g_strconcat ("/core/sparql/", tests[i].test_name, NULL);
 		g_test_add (testpath, TestInfo, &tests[i], setup, test_sparql_query, teardown);
 		g_free (testpath);
