@@ -307,6 +307,10 @@ thread_func (gpointer user_data)
 	started = TRUE;
 	g_main_loop_run (main_loop);
 
+	g_main_loop_unref (main_loop);
+	g_main_context_pop_thread_default (context);
+	g_main_context_unref (context);
+
 	return NULL;
 }
 
